from piecash_utilities.report import report, execute_report


@report(
    title="My simplest report",
    name="piecash-simple-report",
    menu_tip="This simple report ever",
    options_default_section="general",
)
def generate_report(
        book_url,
):
    return "<html><body>Hello world from python !</body></html>"


if __name__ == '__main__':
    execute_report(generate_report)



import hashlib
import inspect
import os
import sys
import traceback

from piecash_utilities.config import get_latest_file
from .options import Option


class Report:
    name = ""
    title = ""
    options = None
    function = None
    menu_tip = ""
    python_script = None

    def __init__(self, name, title, menu_tip, options, options_default_section, function=None, python_script=None):
        self.name = name
        self.title = title
        self.menu_tip = menu_tip

        self.options = [] if options is None else options
        self.options_default_section = options_default_section
        if function:
            self.function = function
        if python_script:
            self.python_script = python_script

    @property
    def guid(self):
        return hashlib.sha224("{}-{}".format(self.name, self.title).encode('utf-8')).hexdigest()

    def generate_scm(self):
        import jinja2

        env = jinja2.Environment(loader=jinja2.PackageLoader(__name__, '.'))
        scm_view = env.get_template("python_report_template.scm").render(
            project=self,
            python_interpreter='"' + repr("'" + sys.executable)[2:].replace("python.exe", "pythonw.exe"),
        )
        return scm_view


def generate_sample_report_python():
    with open(os.path.join(os.path.dirname(__file__), "report_example.py")) as f:
        return f.read()


def generate_sample_report_html():
    with open(os.path.join(os.path.dirname(__file__), "report_example.html")) as f:
        return f.read()


def report(options_default_section,
           title,
           name,
           menu_tip,
           ):
    options = []
    p = Report(options_default_section=options_default_section,
               title=title,
               name=name,
               menu_tip=menu_tip,
               options=options,
               )

    def process_function(f):
        fsig = inspect.signature(f)

        for name, param in fsig.parameters.items():
            if isinstance(param.annotation, Option):
                param.annotation.name = name
                options.append(param.annotation)

        def wrapped():
            dct = {}
            input_options = sys.stdin.readlines()
            for option, option_meta in zip(input_options, options):
                var, value = option.split("|")
                dct[var] = option_meta.parse(value)

            book_url = get_latest_file()
            return f(book_url, **dct)

        wrapped.project = p
        return wrapped

    return process_function


def execute_report(generate_report):
    try:
        s = generate_report()
        print(s)
    except Exception as e:
        # report the trace in html output
        mystdout = os.fdopen(sys.stdout.fileno(), 'w')
        original_write = mystdout.write
        original_write('<html><head><style>pre {font-family: arial;}</style></head><body>')

        def wrapped_write(text):
            text = "".join("<pre>{}</pre>".format(l) for l in text.split("\n"))
            original_write(text)

        mystdout.write = wrapped_write
        traceback.print_exc(file=mystdout)
        original_write("</body></html>")



from .config import get_user_config_path, get_latest_file, update_config_user
from .report import report, DateOption, StringOption, RangeOption, generate_sample_report_python, generate_sample_report_html, execute_report



#!/usr/bin/env python
import os
import sys

from piecash_utilities import generate_sample_report_html
from piecash_utilities import generate_sample_report_python
from piecash_utilities import get_user_config_path


def main():
    if len(sys.argv) <= 1:
        print("provide a name for your report")
        sys.exit(1)
    name_report_base = sys.argv[1]
    name_report = "report_" + name_report_base + ".py"
    name_html =  "report_" + name_report_base + ".html"

    path_report = os.path.join(get_user_config_path(), name_report)
    path_html = os.path.join(get_user_config_path(), name_html)

    if os.path.exists(path_report):
        print("file '{}' already exist, will not overwrite it".format(path_report))
        sys.exit(1)

    with open(path_report, "w") as f:
        f.write(generate_sample_report_python().replace("NAMEOFREPORT", name_report_base))
    with open(path_html, "w") as f:
        f.write(generate_sample_report_html())


    print("file '{}' successfully created".format(path_report))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import glob
import os
import sys

from piecash_utilities import update_config_user, get_user_config_path

if sys.version_info >= (3,0):
    import importlib


    def load_module(file_path):
        return importlib.machinery.SourceFileLoader("mod", file_path).load_module()
else:
    import imp


    def load_module(file_path):
        return imp.load_source("mod", file_path)


def main():
    lines_report = []
    user_path = get_user_config_path()
    for p in glob.glob(os.path.join(user_path, "report_*.py")):
        path_name, ext = os.path.splitext(p)
        name = os.path.basename(path_name)
        mod = load_module(p)

        if hasattr(mod, "generate_report"):
            project = mod.generate_report.project
            project.python_script = name + ext

            scm_view = project.generate_scm()
            scm_name = name + ".scm"
            print("generate", name)
            with open(os.path.join(user_path, scm_name), "w") as f:
                f.write(scm_view)
            lines_report.append('(load (gnc-build-dotgnucash-path "{}"))'.format(scm_name))

    update_config_user(lines_report)


if __name__ == '__main__':
    main()



import os
import traceback

import jinja2
import sys

from piecash_utilities.report import report, RangeOption, DateOption, StringOption, execute_report


@report(
    title="My default example report [NAMEOFREPORT]",
    name="default-sample-NAMEOFREPORT",
    menu_tip="Default sample generated by 'gc_report_create NAMEOFREPORT'",
    options_default_section="main",
)
def generate_report(
        book_url,
        a_number: RangeOption(
            section="main",
            sort_tag="a",
            documentation_string="This is a number",
            default_value=3),
        a_str: StringOption(
            section="main",
            sort_tag="c",
            documentation_string="This is a string",
            default_value="with a default value"),
        a_date: DateOption(
            section="main",
            sort_tag="d",
            documentation_string="This is a date",
            default_value="(lambda () (cons 'absolute (cons (current-time) 0)))"),
        another_number: RangeOption(
            section="main",
            sort_tag="b",
            documentation_string="This is a number",
            default_value=3)
):
    import piecash
    with piecash.open_book(book_url, readonly=True, open_if_lock=True) as b:
        tpl_name = os.path.basename(__file__).replace("py", "html")
        env = jinja2.Environment(loader=jinja2.PackageLoader(__name__, '.'))
        return env.get_template(tpl_name).render(
            enumerate=enumerate,
            list=list,
            path_report=os.path.abspath(__file__),
            **vars()
        )



if __name__ == '__main__':
    execute_report(generate_report)



import os
import sys


def get_latest_file():
    if sys.platform.startswith("win"):
        try:
            import winreg
        except ImportError:
            import _winreg as winreg

        explorer = winreg.OpenKey(
            winreg.HKEY_CURRENT_USER,
            "Software\\GSettings\\org\\gnucash\\history"
        )
        value, type = winreg.QueryValueEx(explorer, "file0")
        return value
    elif sys.platform.startswith("linux"):
        import subprocess
        output_dconf = subprocess.check_output(["dconf", "dump", "/org/gnucash/history/"]).decode()
        from configparser import ConfigParser
        conf = ConfigParser()
        conf.read_string(output_dconf)
        return conf["/"]["file0"][1:-1]
    else:
        raise NotImplemented("not yet implemented for sys.platform = '{}'".format(sys.platform))


def get_user_config_path():
    from os.path import expanduser
    home = expanduser("~")
    if sys.platform.startswith("win") or sys.platform.startswith("linux"):
        return os.path.join(home, ".gnucash")
    else:
        raise NotImplemented("not yet implemented for sys.platform = '{}'".format(sys.platform))


def update_config_user(lines, separator=";; lines automatically added\n;; everything below this line will be scraped"):
    # add the list of lines to the end of the config.user file
    # separating the original content and the new content
    # through a separator
    path = os.path.join(get_user_config_path(), "config.user")
    if os.path.exists(path):
        with open(path, "r") as fin:
            original = fin.read()
    else:
        original = ""

    if separator in original:
        original = original[:original.index(separator)]

    if not original.endswith("\n"):
        original += "\n"

    original += separator + "\n"
    original += "\n".join(lines)

    with open(path, "w") as fout:
        fout.write(original)


# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function

# from builtins import object


# class TestIntegration_ExampleScripts(object):
#     def test_basic(self):
#         pass



from piecash_utilities.report import report, RangeOption, DateOption, StringOption, execute_report


@report(
    title="My simplest report with parameters",
    name="piecash-simple-report-parameters",
    menu_tip="A simple report with parameters",
    options_default_section="general",
)
def generate_report(
        book_url,
        a_number: RangeOption(
            section="main",
            sort_tag="a",
            documentation_string="This is a number",
            default_value=3),
        a_str: StringOption(
            section="main",
            sort_tag="c",
            documentation_string="This is a string",
            default_value="with a default value"),
        a_date: DateOption(
            section="main",
            sort_tag="d",
            documentation_string="This is a date",
            default_value="(lambda () (cons 'absolute (cons (current-time) 0)))"),
        another_number: RangeOption(
            section="main",
            sort_tag="b",
            documentation_string="This is a number",
            default_value=3)
):
    return """<html>
    <body>
        Hello world from python !<br>
        Parameters received:<br>
        <ul>
        <li>a_number = {a_number}</li>
        <li>a_str = {a_str}</li>
        <li>a_date = {a_date}</li>
        <li>another_number = {another_number}</li>
        </ul>
    </body>
    </html>""".format(
        a_str=a_str,
        another_number=another_number,
        a_date=a_date,
        a_number=a_number,
    )


if __name__ == '__main__':
    execute_report(generate_report)



import csv

import click
import piecash


def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()


@click.group()
def import_export():
    pass


@import_export.command(name="export")
@click.argument("gnucash_file", type=click.Path(exists=True, dir_okay=False))
@click.argument("csv_file", type=click.Path(exists=False, dir_okay=False), default="-")
@click.option("--delimiter", "-d", default=";")
@click.option("--date_format", "-df", default="%d/%m/%Y")
@click.option("--warn-if-locked", is_flag=True)
def export_csv(gnucash_file, csv_file, delimiter, date_format, warn_if_locked):
    with click.open_file(csv_file, mode="w") as file:
        writer = csv.DictWriter(file,
                                fieldnames=["guid",
                                            "post_date",
                                            "description",
                                            "notes",
                                            "from_account",
                                            "to_account",
                                            "value",
                                            "currency",
                                            ],
                                delimiter=delimiter.encode())
        writer.writeheader()
        with piecash.open_book(gnucash_file, open_if_lock=not warn_if_locked) as b:
            for tr in b.transactions:
                tr_dct = dict(guid=tr.guid,
                              post_date=tr.post_date.strftime(date_format),
                              description=tr.description,
                              notes=tr.notes)
                splits = tr.splits
                if len(splits) == 2:
                    sp_from = splits[0 if splits[0].value < 0 else 1]
                    sp_to = splits[1 if splits[0].value < 0 else 0]
                    tr_dct.update(dict(from_account=sp_from.account.fullname,
                                       to_account=sp_to.account.fullname,
                                       value=sp_to.value,
                                       currency=tr.currency.mnemonic))
                writer.writerow(tr_dct)


@import_export.command(name="import")
@click.option('--yes', is_flag=True, callback=abort_if_false,
              expose_value=False,
              prompt='Are you sure you want to import transactions in the GnuCash book?')
def import_csv(uri, delimiter, date_format):
    print("This is a test script")


if __name__ == '__main__':
    import_export()



# -*- coding: latin-1 -*-
import pytest

import sys
import codecs

if sys.version_info.major == 2:
    out = codecs.getwriter('UTF-8')(sys.stdout)
else:
    out = sys.stdout

class TestLedger_out_write(object):
    def test_out_write(self):
        print("ok")



import piecash

from piecash_utilities.report import report, execute_report


@report(
    title="My simplest report with a book",
    name="piecash-simple-report-book",
    menu_tip="A simple report that opens a book",
    options_default_section="general",
)
def generate_report(
        book_url,
):
    with piecash.open_book(book_url, readonly=True, open_if_lock=True) as book:
        return """<html>
        <body>
            Hello world from python !<br>
            Book : {book_url}<br>
            List of accounts : {accounts}
        </body>
        </html>""".format(
            book_url=book_url,
            accounts=[acc.fullname for acc in book.accounts],
        )


if __name__ == '__main__':
    execute_report(generate_report)



# coding=utf-8
from __future__ import print_function

import io
import os.path

import click
import jinja2
import sys
from piecash import open_book


@click.group()
def import_export():
    pass


@import_export.command(name="export")
@click.argument("gnucash_file", type=click.Path(exists=True, dir_okay=False))
@click.argument("xml_file", type=click.Path(exists=False, dir_okay=False), default="-")
@click.option("--warn-if-locked", is_flag=True)
def export_norme_A47(gnucash_file, xml_file, warn_if_locked):
    with open_book(gnucash_file, open_if_lock=not warn_if_locked) as book:
        # could add some filtering based on post_date if required
        transactions = book.transactions

        env = jinja2.Environment(trim_blocks=True, lstrip_blocks=True)
        xml = env.from_string(u"""
<?xml version="1.0"?>
<comptabilite>
  <exercice>
    <DateCloture>2016-12-31T00:00:00</DateCloture>
    <journal>
      <JournalCode>Le code du Journal</JournalCode>
      <JournalLib>Le libellé du Journal</JournalLib>
      {% for i, ecriture in enumerate(transactions) %}
      <ecriture>
        <EcritureNum>{{ i }}</EcritureNum>
        <EcritureDate>{{ ecriture.post_date.strftime("%Y-%m-%d") }}</EcritureDate>
        <EcritureLib>{{ ecriture.description }}</EcritureLib>
        <PieceRef>{{ ecriture.num }}</PieceRef>
        <PieceDate>{{ ecriture.post_date.strftime("%Y-%m-%d") }}</PieceDate>
        <ValidDate>{{ ecriture.post_date.strftime("%Y-%m-%d") }}</ValidDate>
        {% for sp in ecriture.splits %}
        <ligne>
          <CompteNum>{{ sp.account.code }}</CompteNum>
          <CompteLib>{{ sp.account.name }}</CompteLib>
          <CompteAuxNum>Le numéro de compte auxiliaire (à blanc si non utilisé)</CompteAuxNum>
          <CompteAuxLib>Le libellé de compte auxiliaire (à blanc si non utilisé)</CompteAuxLib>
          <Montantdevise></Montantdevise>
          <Montant>{{ abs(sp.value) }}</Montant>
          <Sens>{% if sp.value >0 %}c{% else %}d{% endif %}</Sens>
        </ligne>
        {% endfor %}
      </ecriture>
      {% endfor %}
    </journal>
  </exercice>
</comptabilite>
        """).render(transactions=transactions,
                    enumerate=enumerate,
                    abs=abs,
                    )

        if xml_file and xml_file!="-":
            with io.open(xml_file, "w", encoding="utf-8") as f:
                f.write(xml)
        else:
            sys.stdout.write(xml)


if __name__ == '__main__':
    import_export()



import ast
import datetime

import jinja2


class Option:
    """An option to be used in a report

    Attributes:
        type (str): the type of the option of the form 'gnc:make-number-range-option'
        section (str): the section/tab where the option should appear in the option dialog
        sort_tag (str): a string defining the sort order in the tab
        documentation_string (str): the doc string of the option
        default_value (str): the default value of the option
        name (str): the name of the variable
    """
    def __init__(self, type, section, sort_tag, documentation_string, default_value, name=None):
        self.type = type
        self.section = section
        self.name = name
        self.sort_tag = sort_tag
        self.documentation_string = documentation_string
        self.default_value = default_value

    def render_scheme(self):
        pass

    def render_serialise(self):
        return jinja2.Template("""
            (op-value "{{option.section}}" "{{option.name}}")""").render(option=self)

    def parse(self, value):
        return ast.literal_eval(value)


class DateOption(Option):
    def __init__(self, is_datetime=False, **kwargs):
        super(DateOption, self).__init__(type="gnc:make-date-option", **kwargs)
        self.is_datetime = is_datetime

    def render_serialise(self):
        return jinja2.Template("""(cadr (op-value "{{option.section}}" "{{option.name}}"))""").render(option=self)

    def parse(self, value):
        return datetime.datetime.fromtimestamp(ast.literal_eval(value))

    def render_scheme(self):
        return jinja2.Template("""    (add-option
       ({{ option.type }}
      (N_ "{{option.section}}") (N_ "{{option.name}}")
      "{{option.sort_tag}}" (N_ "{{option.documentation_string}}")
      {{option.default_value}}  ;; default
      #f 'absolute #f
      ))
        """).render(option=self)


class RangeOption(Option):
    def __init__(self, lower=0, upper=10000, decimals=2, step_size=0.01, **kwargs):
        super(RangeOption, self).__init__(type="gnc:make-number-range-option", **kwargs)
        self.lower = lower
        self.upper = upper
        self.decimals = decimals
        self.step_size = step_size

    def render_scheme(self):
        return jinja2.Template("""(add-option
       ({{ option.type }}
      (N_ "{{option.section}}") (N_ "{{option.name}}")
      "{{option.sort_tag}}" (N_ "{{option.documentation_string}}")
      {{option.default_value}}  ;; default
      {{option.lower }}     ;; lower bound
      {{option.upper }} ;; upper bound
      {{option.decimals }}     ;; number of decimals
      {{option.step_size }}    ;; step size
      ))
        """).render(option=self)


class StringOption(Option):
    def __init__(self, **kwargs):
        super(StringOption, self).__init__(type="gnc:make-string-option", **kwargs)

    def render_scheme(self):
        return jinja2.Template("""    (add-option
       ({{ option.type }}
      (N_ "{{option.section}}") (N_ "{{option.name}}")
      "{{option.sort_tag}}" (N_ "{{option.documentation_string}}")
      "{{option.default_value}}"  ;; default
      ))
        """).render(option=self)

    def parse(self, value):
        return value
